@@ -13,7 +13,13 @@ import redis
 from prompt_toolkit.shortcuts import clear
 from prompt_toolkit.formatted_text import FormattedText
 from redis.connection import Connection, SSLConnection, UnixDomainSocketConnection
-from redis.exceptions import AuthenticationError, ConnectionError, TimeoutError
+from redis.exceptions import (
+    AuthenticationError,
+    ConnectionError,
+    TimeoutError,
+    ResponseError,
+)
+
 
 from . import markdown, renders
 from .data import commands as commands_data
@@ -27,13 +33,14 @@ from .commands import (
 )
 from .completers import IRedisCompleter
 from .config import config
-from .exceptions import NotRedisCommand, InvalidArguments, AmbiguousCommand
+from .exceptions import NotRedisCommand, InvalidArguments, AmbiguousCommand, NotSupport
 from .renders import OutputRender
 from .utils import (
     compose_command_syntax,
     nativestr,
     exit,
     convert_formatted_text_to_bytes,
+    parse_url,
 )
 from .warning import confirm_dangerous_command
 
@@ -55,6 +62,7 @@ class Client:
         path=None,
         scheme="redis",
         username=None,
+        client_name=None,
     ):
         self.host = host
         self.port = port
@@ -62,33 +70,19 @@ class Client:
         self.path = path
         # FIXME username is not using...
         self.username = username
+        self.client_name = client_name
         self.scheme = scheme
 
-        if scheme in ("redis", "rediss"):
-            connection_kwargs = {
-                "host": host,
-                "port": port,
-                "db": db,
-                "password": password,
-                "socket_keepalive": config.socket_keepalive,
-            }
-            if scheme == "rediss":
-                connection_class = SSLConnection
-            else:
-                connection_class = Connection
-        else:
-            connection_kwargs = {"db": db, "password": password, "path": path}
-            connection_class = UnixDomainSocketConnection
-
-        if config.decode:
-            connection_kwargs["encoding"] = config.decode
-            connection_kwargs["decode_responses"] = True
-            connection_kwargs["encoding_errors"] = "replace"
-
-        logger.debug(
-            f"connection_class={connection_class}, connection_kwargs={connection_kwargs}"
+        self.connection = self.create_connection(
+            host,
+            port,
+            db,
+            password,
+            path,
+            scheme,
+            username,
+            client_name=client_name,
         )
-        self.connection = connection_class(**connection_kwargs)
 
         # all command upper case
         self.answer_callbacks = command2callback
@@ -110,6 +104,50 @@ class Client:
         if config.version and re.match(r"([\d\.]+)", config.version):
             self.auth_compat(config.version)
 
+    def create_connection(
+        self,
+        host=None,
+        port=None,
+        db=0,
+        password=None,
+        path=None,
+        scheme="redis",
+        username=None,
+        client_name=None,
+    ):
+        if scheme in ("redis", "rediss"):
+            connection_kwargs = {
+                "host": host,
+                "port": port,
+                "db": db,
+                "password": password,
+                "socket_keepalive": config.socket_keepalive,
+                "client_name": client_name,
+            }
+            if scheme == "rediss":
+                connection_class = SSLConnection
+            else:
+                connection_class = Connection
+        else:
+            connection_kwargs = {
+                "db": db,
+                "password": password,
+                "path": path,
+                "client_name": client_name,
+            }
+            connection_class = UnixDomainSocketConnection
+
+        if config.decode:
+            connection_kwargs["encoding"] = config.decode
+            connection_kwargs["decode_responses"] = True
+            connection_kwargs["encoding_errors"] = "replace"
+
+        logger.debug(
+            f"connection_class={connection_class}, connection_kwargs={connection_kwargs}"
+        )
+
+        return connection_class(**connection_kwargs)
+
     def auth_compat(self, redis_version: str):
         with_username = StrictVersion(redis_version) >= StrictVersion("6.0.0")
         if with_username:
@@ -166,10 +204,19 @@ class Client:
         if command == "EXIT":
             exit()
 
-    def execute(self, command_name, *args, **options):
+    def execute(self, *args, **kwargs):
+        logger.info(
+            f"execute: connection={self.connection} args={args}, kwargs={kwargs}"
+        )
+        return self.execute_by_connection(self.connection, *args, **kwargs)
+
+    def execute_by_connection(self, connection, command_name, *args, **options):
         """Execute a command and return a parsed response
         Here we retry once for ConnectionError.
         """
+        logger.info(
+            f"execute by connection: connection={connection}, name={command_name}, {args}, {options}"
+        )
         retry_times = config.retry_times  # FIXME configureable
         last_error = None
         need_refresh_connection = False
@@ -181,11 +228,12 @@ class Client:
                         f"{str(last_error)} retrying... retry left: {retry_times+1}",
                         file=sys.stderr,
                     )
-                    self.connection.disconnect()
-                    self.connection.connect()
-                    logger.info(f"New connection created, retry on {self.connection}.")
-                self.connection.send_command(command_name, *args)
-                response = self.connection.read_response()
+                    connection.disconnect()
+                    connection.connect()
+                    logger.info(f"New connection created, retry on {connection}.")
+                logger.info(f"send_command: {command_name} , {args}")
+                connection.send_command(command_name, *args)
+                response = connection.read_response()
             except AuthenticationError:
                 raise
             except (ConnectionError, TimeoutError) as e:
@@ -193,6 +241,13 @@ class Client:
                 last_error = e
                 retry_times -= 1
                 need_refresh_connection = True
+            except (ResponseError) as e:
+                response_message = str(e)
+                if response_message.startswith("MOVED"):
+                    return self.reissue_with_redirect(
+                        response_message, command_name, *args, **options
+                    )
+                raise e
 
             except redis.exceptions.ExecAbortError:
                 config.transaction = False
@@ -201,6 +256,45 @@ class Client:
                 return response
         raise last_error
 
+    def reissue_with_redirect(self, response, *args, **kwargs):
+        """
+        For redis cluster, when server response a "MOVE ..." response, we auto-
+        redirect to the target node, reissue the original command.
+
+        This feature is not supported for unix socket connection.
+        """
+        # Redis Cluster only supports database zero.
+        _, slot, ip_port = response.split(" ")
+        ip, port = ip_port.split(":")
+        port = int(port)
+
+        print(response, file=sys.stderr)
+
+        connection = self.create_connection(ip, port)
+        # if user sets dsn for dest node
+        # use username and password from dsn settings
+        if config.alias_dsn:
+            for dsn_name, dsn_url in config.alias_dsn.items():
+                dsn = parse_url(dsn_url)
+                if dsn.host == ip and dsn.port == port:
+                    print(
+                        f"Connect {ip}:{port} via dns settings of {dsn_name}",
+                        file=sys.stderr,
+                    )
+                    connection = self.create_connection(
+                        dsn.host,
+                        dsn.port,
+                        dsn.db,
+                        dsn.password,
+                        dsn.path,
+                        dsn.scheme,
+                        dsn.username,
+                    )
+                    break
+
+        connection.connect()
+        return self.execute_by_connection(connection, *args, **kwargs)
+
     def render_response(self, response, command_name):
         "Parses a response from the Redis server"
         logger.info(f"[Redis-Server] Response: {response}")
@@ -254,7 +348,7 @@ class Client:
         grammar = completer.get_completer(input_text=rawinput).compiled_grammar
         matched = grammar.match(rawinput)
         if not matched:
-            # invalide command!
+            # invalid command!
             return rawinput, None
         variables = matched.variables()
         shell_command = variables.get("shellcommand")
@@ -380,6 +474,8 @@ class Client:
 
         Only works when compile-grammar thread is done.
         """
+        if command_name.upper() == "HELLO":
+            raise NotSupport("IRedis currently not support RESP3, sorry about that.")
         # TRANSATION state chage
         if command_name.upper() in ["EXEC", "DISCARD"]:
             logger.debug(f"[After hook] Command is {command_name}, unset transaction.")
@@ -400,7 +496,7 @@ class Client:
         redis_grammar = completer.get_completer(command).compiled_grammar
         m = redis_grammar.match(command)
         if not m:
-            # invalide command!
+            # invalid command!
             return
         variables = m.variables()
         # zset withscores
@@ -415,7 +511,7 @@ class Client:
             doc = read_text(commands_data, f"{command_docs_name}.md")
         except FileNotFoundError:
             raise NotRedisCommand(
-                f"{command_summary_name} is not a valide Redis command."
+                f"{command_summary_name} is not a valid Redis command."
             )
         rendered_detail = markdown.render(doc)
         summary_dict = commands_summary[command_summary_name]
