@@ -27,9 +27,17 @@ class OutputRender:
     @staticmethod
     def get_render(command_name):
         """Dynamic render output due to command name."""
-        command_upper = command_name.upper()
+        command_upper = " ".join(command_name.split()).upper()
         callback_name = command2callback.get(command_upper)
-        callback = getattr(OutputRender, callback_name)
+
+        # using `render_list_or_string` as default render.
+        if callback_name is None:
+            callback = OutputRender.render_list_or_string
+        else:
+            callback = getattr(
+                OutputRender, callback_name, OutputRender.render_list_or_string
+            )
+
         logger.info(
             f"[render] Find callback {callback_name}, for command: {command_name}"
         )
@@ -70,7 +78,7 @@ class OutputRender:
     @staticmethod
     def render_nested_pair(value):
         """
-        For redis internel responses.
+        For redis internal responses.
         Always decode with utf-8
         Render nested list.
         Items come as pairs.
@@ -379,7 +387,7 @@ def _render_scan(render_response, response):
 
     rendered = [
         ("class:type", "(cursor) "),
-        ("class:integer", cursor.decode()),
+        ("class:integer", cursor if isinstance(cursor, str) else cursor.decode()),
         ("", "\n"),
     ]
     rendered_keys = render_response(responses)
